//========================================================================================
//
//	File:				ShpTrakr.cpp
//	Release Version:	$ 1.0d11 $
//
//	Author:				Henri Lamiraux
//
//	Copyright:	 1993, 1995 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "ODFDraw.hpp"

#ifndef SHPTRAKR_H
#include "ShpTrakr.h"
#endif

#ifndef UTILS_H
#include "Utils.h"
#endif

#ifndef BASESHP_H
#include "BaseShp.h"
#endif

#ifndef BOUNDSHP_H
#include "BoundShp.h"
#endif

#ifndef LINESHP_H
#include "LineShp.h"
#endif

#ifndef OVALSHP_H
#include "OvalShp.h"
#endif

#ifndef RECTSHP_H
#include "RectShp.h"
#endif

#ifndef RRECTSHP_H
#include "RRectShp.h"
#endif

#ifndef TEXTSHP_H
#include "TextShp.h"
#endif

// ----- Part Layer -----

#ifndef FWUTIL_H
#include "FWUtil.h"
#endif

#ifndef FWVIEW_H
#include "FWView.h"
#endif

// ----- OS Layer -----

#ifndef FWRECSHP_H
#include "FWRecShp.h"
#endif

//========================================================================================
// Runtime Information
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment odfdraw
#endif

//========================================================================================
//	class CShapeTracker
//========================================================================================

//----------------------------------------------------------------------------------------
//	CShapeTracker::CShapeTracker
//----------------------------------------------------------------------------------------

CShapeTracker::CShapeTracker(Environment* ev, FW_CView* view, ODFacet* facet, CBaseShape* theShape, FW_Boolean gridOn) :
	FW_CTracker(ev, view, facet),
	fShape(theShape),
	fErase(FALSE),
	fGridOn(gridOn)
{
}

//----------------------------------------------------------------------------------------
//	CShapeTracker::~CShapeTracker
//----------------------------------------------------------------------------------------

CShapeTracker::~CShapeTracker()
{
}

//----------------------------------------------------------------------------------------
//	CShapeTracker::BeginTracking
//----------------------------------------------------------------------------------------

FW_CPoint CShapeTracker::BeginTracking(Environment* ev, 
									const FW_CPoint& anchorPoint)
{
	FW_CPoint point(anchorPoint);
	ForceToGrid(ev, point);
	return point;
}

//----------------------------------------------------------------------------------------
//	CShapeTracker::ContinueTracking
//----------------------------------------------------------------------------------------

FW_CPoint CShapeTracker::ContinueTracking(Environment* ev,
										const FW_CPoint& anchorPoint, 
										const FW_CPoint& previousPoint, 
										const FW_CPoint& currentPoint)
{
	FW_CPoint curLoc(currentPoint);
	
	ForceToGrid(ev, curLoc);
	
	if (previousPoint != curLoc)
	{
		FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
		
		if (fErase)
			fShape->TrackFeedback(ev, GetFacet(ev), vc, anchorPoint, previousPoint, TRUE);
		
		fShape->TrackFeedback(ev, GetFacet(ev), vc, anchorPoint, curLoc, FALSE);
		
		fErase = TRUE;
	}
	
	return curLoc;
}

//----------------------------------------------------------------------------------------
//	CShapeTracker::EndTracking
//----------------------------------------------------------------------------------------

FW_Boolean CShapeTracker::EndTracking(Environment* ev,
									const FW_CPoint& anchorPoint, 
									const FW_CPoint& lastPoint)
{
	if (fErase)
	{
		FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
		
		fShape->TrackFeedback(ev, GetFacet(ev), vc, anchorPoint, lastPoint, TRUE);
	}
	
	return anchorPoint != lastPoint;
}

//----------------------------------------------------------------------------------------
//	CShapeTracker::ForceToGrid
//----------------------------------------------------------------------------------------

void CShapeTracker::ForceToGrid(Environment* ev, FW_CPoint& point)
{
	if (fGridOn)
	{
		point.x.SetInt((point.x.AsInt() + 5) / 8 * 8);
		point.y.SetInt((point.y.AsInt() + 5) / 8 * 8);
	}
}

//========================================================================================
//	class CResizeTracker
//========================================================================================

//----------------------------------------------------------------------------------------
//	CResizeTracker::CResizeTracker
//----------------------------------------------------------------------------------------

CResizeTracker::CResizeTracker(Environment* ev,
								FW_CView* view, ODFacet* facet, 
								CBaseShape* theShape, short whichHandle,
								const FW_PInk& resizeInk, const FW_PStyle& resizeStyle,
								FW_Boolean gridOn) :
	FW_CTracker(ev, view, facet),
	fShape(theShape),
	fWhichHandle(whichHandle),
	fResizeInk(resizeInk),
	fResizeStyle(resizeStyle),
	fErase(FALSE),
	fGridOn(gridOn)
{
}

//----------------------------------------------------------------------------------------
//	CResizeTracker::~CResizeTracker
//----------------------------------------------------------------------------------------

CResizeTracker::~CResizeTracker()
{
}

//----------------------------------------------------------------------------------------
//	CResizeTracker::BeginTracking
//----------------------------------------------------------------------------------------

FW_CPoint CResizeTracker::BeginTracking(Environment* ev, const FW_CPoint& anchorPoint)
{
	FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
	
	FW_CRectShape handle(FW_kZeroRect, FW_kFill);
	handle.SetInk(FW_kInvertInk);
	fShape->CalcHandle(fWhichHandle, &handle);
	handle.Render(vc);	// redraw the handle

	FW_CPoint point(anchorPoint);
	ForceToGrid(ev, point);
	
	FW_CPoint handleCenter;
	fShape->GetHandleCenter(fWhichHandle, handleCenter);
	fDelta = point - handleCenter;

	return handleCenter;
}

//----------------------------------------------------------------------------------------
//	CResizeTracker::ContinueTracking
//----------------------------------------------------------------------------------------

FW_CPoint CResizeTracker::ContinueTracking(Environment* ev,
											const FW_CPoint& anchorPoint, 
											const FW_CPoint& previousPoint, 
											const FW_CPoint& currentPoint)
{
FW_UNUSED(anchorPoint);

	// ----- Adjust for the size of the handle
	FW_CPoint curLoc(currentPoint - fDelta);		
	ForceToGrid(ev, curLoc);

	if (previousPoint != curLoc)
	{
		FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
	
		if (fErase)
			fShape->ResizeFeedback(vc, fResizeInk, fResizeStyle, fWhichHandle, previousPoint);		// erase
				
		fShape->ResizeFeedback(vc, fResizeInk, fResizeStyle, fWhichHandle, curLoc);			// draw
		
		fErase = TRUE;				
	}
	
	return curLoc;
}

//----------------------------------------------------------------------------------------
//	CResizeTracker::EndTracking
//----------------------------------------------------------------------------------------

FW_Boolean CResizeTracker::EndTracking(Environment* ev,
									const FW_CPoint& anchorPoint, 
									const FW_CPoint& lastPoint)
{
FW_UNUSED(anchorPoint);

	fLastLocation = lastPoint - fDelta;
	ForceToGrid(ev, fLastLocation);
	
	if (fErase)
	{
		FW_CViewContext vc(ev, GetView(ev), GetFacet(ev));
	
		fShape->ResizeFeedback(vc, fResizeInk, fResizeStyle, fWhichHandle, fLastLocation);
	}
	
	return anchorPoint != lastPoint;
}

//----------------------------------------------------------------------------------------
//	CResizeTracker::ForceToGrid
//----------------------------------------------------------------------------------------

void CResizeTracker::ForceToGrid(Environment* ev, FW_CPoint& point)
{
	if (fGridOn)
	{
		point.x.SetInt((point.x.AsInt() + 5) / 8 * 8);
		point.y.SetInt((point.y.AsInt() + 5) / 8 * 8);
	}
}
